package pong;

import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.security.InvalidParameterException;

import appia.AppiaEventException;
import appia.Channel;
import appia.Direction;

import pong.events.ButtonStartEvent;
import pong.events.MouseMovedEvent;

/**
 * Classe responsvel por desenhar todos os objectos no ecran e tambm
 * responsvel por gerar eventos para um canal Appia aquando o movimento do
 * rato. Quando o utilizador interage com a aplicao usando o rato,  enviado
 * um evento para um canal Appia dado no construtor desta classe.
 * <p>
 * ATENO: No  necessrio criar uma instncia desta classe directamente.
 * Isto dever ser efectuado pela API fornecida.
 * </p>
 * 
 * @author nuno
 */
public class PongGraphics extends javax.swing.JFrame {

	private static final String TITLE = "Fault Tolerant Pong";

	private Channel channel;

	private PongCanvas canvas1;
	private java.awt.Canvas canvas3;
	private javax.swing.JButton jButton1;
	private javax.swing.JLabel[] jLabel;

	private Ball ball;
	private Paddle[] paddles;
	private Thread timer;
	private int[] pontuacao;
	private int myPaddle;
	private Point currentPoint;

	/**
	 * Cria uma nova intancia da interface e desenha-a no ecran.
	 * 
	 * @param c
	 *                   Canal Appia
	 * @param myP
	 *                   nmero do jogador (ou Paddle)
	 */
	public PongGraphics(Channel c, int myP) {
		channel = c;
		myPaddle = myP;
		pontuacao = new int[4];
		for (int i = 0; i < pontuacao.length; i++)
			pontuacao[i] = -1;
		initComponents();
		canvas1.createBufferStrategy(2);
		// TODO: verificar se isto serve!!!
		currentPoint = new Point(canvas1.getX(),canvas1.getY());

		paddles = new Paddle[4];
		paddles[0] = new Paddle(Paddle.VERTICAL, Paddle.LEFT, canvas1);
		paddles[1] = new Paddle(Paddle.VERTICAL, Paddle.RIGHT, canvas1);
		paddles[2] = new Paddle(Paddle.HORIZONTAL, Paddle.TOP, canvas1);
		paddles[3] = new Paddle(Paddle.HORIZONTAL, Paddle.BOTTOM, canvas1);
		canvas1.setPaddles(paddles);
		show();
	}

	protected void createBall() {
		if (ball != null)
			return;
		ball =
			new Ball(
				canvas1.getX(),
				canvas1.getY(),
				canvas1.getWidth(),
				canvas1.getHeight(),
				paddles);
		canvas1.setBall(ball);
	}

	protected void createBall(Point initialPos) {
		if (ball != null)
			return;
		ball =
			new Ball(
				initialPos,
				canvas1.getX(),
				canvas1.getY(),
				canvas1.getWidth(),
				canvas1.getHeight(),
				paddles);
		canvas1.setBall(ball);
	}

	protected void setWindowTitle(int player) {
		setTitle(TITLE + " - Player " + player);
	}

	protected Ball getBall() {
		return ball;
	}

	protected Paddle getPaddle(int player) {
		if (player <= 0 || player > 4)
			throw new InvalidParameterException("Range of Paddles is between 1 and 4.");
		return paddles[player - 1];
	}

	protected PongCanvas getCanvas() {
		return canvas1;
	}

	protected int getPontuation(int player) {
		return pontuacao[player - 1];
	}

	private void initComponents() {
		canvas1 = new PongCanvas(ball, paddles);
		jButton1 = new javax.swing.JButton();
		canvas3 = new java.awt.Canvas();
		jLabel = new javax.swing.JLabel[4];
		jLabel[0] = new javax.swing.JLabel();
		jLabel[1] = new javax.swing.JLabel();
		jLabel[2] = new javax.swing.JLabel();
		jLabel[3] = new javax.swing.JLabel();

		getContentPane().setLayout(
			new org.netbeans.lib.awtextra.AbsoluteLayout());

		setTitle(TITLE);
		setResizable(false);
		/*
		 * Metodo invocado quando a janela  fechada. Neste caso, a aplicao
		 * termina.
		 */
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});

		canvas1.setBackground(new java.awt.Color(0, 153, 204));
		canvas1.setFocusable(true);
		canvas1.setForeground(new java.awt.Color(255, 255, 0));
		/*
		 * metodo executado quando o utilizador move o rato sobre o canvas
		 */
		canvas1
			.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
			public void mouseMoved(java.awt.event.MouseEvent evt) {
				canvas1MouseMoved(evt);
			}
		});

		canvas1.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {canvas1KeyPressed(e);}
			public void keyReleased(KeyEvent e) {}
			public void keyTyped(KeyEvent e) {}
		});

//		addKeyListener(new KeyListener() {
//			public void keyPressed(KeyEvent e) {
//				System.out.println("Tecla "+e.getKeyChar());
//				canvas1KeyPressed(e);
//			}
//			public void keyReleased(KeyEvent e) {
//			}
//			public void keyTyped(KeyEvent e) {
//			}
//		});

		getContentPane().add(
			canvas1,
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				10,
				10,
				400,
				400));

		jButton1.setText("Start");
		jButton1.setToolTipText("Start playing game.");
		/*
		 * metodo executado quando o utilizador clica no Start.
		 */
		jButton1.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				jButton1MouseClicked(evt);
			}
		});

		getContentPane().add(
			jButton1,
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				420,
				150,
				-1,
				-1));

		getContentPane().add(
			canvas3,
			new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 410, 520, 10));

		updatePontuation(1, 0);
		getContentPane().add(
			jLabel[0],
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				420,
				220,
				100,
				20));

		updatePontuation(2, 0);
		getContentPane().add(
			jLabel[1],
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				420,
				240,
				100,
				20));

		updatePontuation(3, 0);
		getContentPane().add(
			jLabel[2],
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				420,
				260,
				100,
				20));

		updatePontuation(4, 0);
		getContentPane().add(
			jLabel[3],
			new org.netbeans.lib.awtextra.AbsoluteConstraints(
				420,
				280,
				100,
				20));

		pack();
	}

	private void jButton1MouseClicked(java.awt.event.MouseEvent evt) {
		canvas1.requestFocus();
		if (getPaddle(myPaddle).isActive())
			return;
		ButtonStartEvent btEvent = new ButtonStartEvent();
		btEvent.setPaddle(myPaddle);
		try {
			btEvent.asyncGo(channel, Direction.DOWN);
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}

	}

	private void canvas1MouseMoved(java.awt.event.MouseEvent evt) {
		MouseMovedEvent mouseEvent = new MouseMovedEvent();
		mouseEvent.setPaddle(myPaddle);
		mouseEvent.setPoint(evt.getPoint());
		currentPoint = evt.getPoint();
		try {
			mouseEvent.asyncGo(channel, Direction.DOWN);
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	private void canvas1KeyPressed(KeyEvent e) {
		switch (e.getKeyCode()) {
			case KeyEvent.VK_UP :
				if(currentPoint.y>canvas1.getY())
					currentPoint.move(currentPoint.x,currentPoint.y-10);
				break;
			case KeyEvent.VK_DOWN :
				currentPoint.move(currentPoint.x,currentPoint.y+10);
				break;
			case KeyEvent.VK_LEFT :
				if(currentPoint.x>canvas1.getX())
					currentPoint.move(currentPoint.x-10,currentPoint.y);
				break;
			case KeyEvent.VK_RIGHT :
				currentPoint.move(currentPoint.x+10,currentPoint.y);
				break;
			default:
			}

		MouseMovedEvent mouseEvent = new MouseMovedEvent();
		mouseEvent.setPaddle(myPaddle);
		mouseEvent.setPoint((Point) currentPoint.clone());
		try {
			mouseEvent.asyncGo(channel, Direction.DOWN);
		} catch (AppiaEventException ex) {
			ex.printStackTrace();
		}

	}
	/** Exit the Application */
	private void exitForm(java.awt.event.WindowEvent evt) {
		System.exit(0);
	}

	protected void updatePontuation(int player, int newPont) {
		pontuacao[player - 1] = newPont;
		String text = "Player" + player + " - " + pontuacao[player - 1];
		jLabel[player - 1].setText(text);
	}

}
