/*
 * Created on 16-Sep-2003
 */
package test;

import java.net.InetAddress;
import java.net.UnknownHostException;

import appia.Appia;
import appia.AppiaCursorException;
import appia.AppiaDuplicatedSessionsException;
import appia.AppiaInvalidQoSException;
import appia.Channel;
import appia.ChannelCursor;
import appia.Layer;
import appia.QoS;

import appia.protocols.tcpcomplete.TcpCompleteLayer;
import appia.protocols.udpsimple.InetWithPort;

/**
 * @author nuno
 */
public class ApplTester {

	public static void main(String[] args) {
		/*
		 * Argumentos...
		 */
		if(args.length != 4)
			invalidArgs();
		int localPort = Integer.parseInt(args[0]);
		String remoteHost = args[1];
		int remotePort = Integer.parseInt(args[2]);
		int player = Integer.parseInt(args[3]);
		
		/*
		 * Criao do array de Layers.
		 * Note-se que o indice 0  a camada mais baixa.
		 */
		Layer[] qos = {
			//new UdpSimpleLayer(),
			//new FifoLayer(),
			new TcpCompleteLayer(),
			//new NewTcpLayer(),
			new ApplLayer()
		}; 
		
		/*
		 * Criao da QoS do canal. 
		 */
		QoS myQoS = null;
		try {
			myQoS = new QoS("Test pong QoS",qos);
		} catch (AppiaInvalidQoSException e) {
			e.printStackTrace();
		}
		
		/*
		 * Criao do canal. Neste momento, o canal ainda no comeou a trabalhar.
		 */
		Channel channel = myQoS.createUnboundChannel("Test pong Channel");
		
		/*
		 * Neste caso, precisei de criar uma das sesses antes de inicializar o canal
		 * para lhe passar parametros.
		 */
		ApplSession session = (ApplSession) qos[qos.length-1].createSession();
		try {
			/*
			 * Aqui est a passagem dos parametros.
			 */
			session.init(localPort,new InetWithPort(InetAddress.getByName(remoteHost),remotePort),player);
		} catch (UnknownHostException e2) {
			e2.printStackTrace();
		}
		/*
		 * O cursor serve para colocar sesses no canal.
		 */
		ChannelCursor cc = channel.getCursor();
		/*
		 * neste caso, quero que a sesso que criei fique no topo.
		 */
		cc.top();
		try {
			/*
			 * Atribuio da sesso.
			 */
			cc.setSession(session);
		} catch (AppiaCursorException e3) {
			e3.printStackTrace();
		}
		/*
		 * As sesses que no so colocadas antes de se inicializar o canal,
		 * so criadas automaticamente pelo Appia. 
		 */
		try {
			channel.start();
		} catch (AppiaDuplicatedSessionsException e1) {
			e1.printStackTrace();
		}
		/*
		 * A partir daqui, o Appia comea a funcionar.
		 * Isto  um ciclo infinito, que no deve ser bloqueado em 
		 * nenhuma camada do Appia.
		 * Quando o Appia invoca o handle() de cada camada, esta deve
		 * sempre executar algum processamento e retornar.
		 */
		Appia.run();
	}

	/**
	 * 
	 */
	private static void invalidArgs() {
		System.out.println("Invalid number of arguments. Usage:");
		System.out.println("java ApplTester <local_port> <remote_host> <remote_port> <1|2>");
		System.exit(1);
	}
}
